package main

import (
	"errors"
	"html/template"
	"log"
	"net/http"

	"github.com/masterminds/sprig"
)

const (
	addr = ":8080"
)

type data map[string]any

/*
@app.route("/")
def index():
*/
func index(w http.ResponseWriter, r *http.Request) {
	http.Redirect(w, r, "/contacts", http.StatusSeeOther)
}

/*
@app.route("/contacts")
*/
func contacts(cs *ContactsStore) http.HandlerFunc {
	tpl := makeTemplate(
		"layout",
		"index",
	)
	return func(w http.ResponseWriter, r *http.Request) {
		search := r.URL.Query().Get("q")
		var contacts_set []Contact
		if search == "" {
			contacts_set = cs.GetAll()
		} else {
			contacts_set = cs.Get(search)
		}
		if err := tpl.ExecuteTemplate(w, "layout.html", data{
			"contacts": contacts_set,
			"search":   search,
		}); err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
		}
	}
}

func makeTemplate(first string, others ...string) *template.Template {
	paths := append([]string{first}, others...)
	for i, path := range paths {
		paths[i] = "./templates/" + path + ".gohtml"
	}
	tpl := template.Must(template.ParseFiles(paths...)).
		Funcs(sprig.FuncMap())
	return tpl
}

func setupRoutes(mux *http.ServeMux, cs *ContactsStore) {
	mux.HandleFunc("/", index)
	mux.HandleFunc("/favicon.ico", func(w http.ResponseWriter, r *http.Request) {
		http.ServeFile(w, r, "./static/img/favicon.ico")
	})
	mux.Handle("/static/", http.StripPrefix("/static/", http.FileServer(http.Dir("./static"))))
	mux.Handle("/contacts", contacts(cs))
}

func main() {
	cs, err := NewContactsStore()
	if err != nil {
		log.Fatalf("initializing contacts: %v\n", err)
	}
	mux := http.NewServeMux()
	setupRoutes(mux, cs)
	log.Printf("Listening on http://localhost%s", addr)
	if err := http.ListenAndServe(addr, mux); err != nil {
		if !errors.Is(err, http.ErrServerClosed) {
			log.Printf("Server error: %v\n", err)
			return
		}
	}
	log.Println("Server shutdown")
}
